package at.acdp.urweb.web;

import java.nio.file.Paths;

import at.acdp.urweb.URBot;
import at.acdp.urweb.sclient.URLog;
import io.undertow.Handlers;
import io.undertow.Undertow;
import io.undertow.server.HttpHandler;
import io.undertow.server.HttpServerExchange;
import io.undertow.server.handlers.HttpTraceHandler;
import io.undertow.server.handlers.resource.PathResourceManager;
import org.slf4j.LoggerFactory;

import static io.undertow.Handlers.resource;

public class WebServer {
    private final static org.slf4j.Logger logger = LoggerFactory.getLogger(WebServer.class);
    private final int port;
    private final URBot urbot;
    private Undertow server;

    public WebServer(int port, boolean debug, URBot urbot) {
        this.urbot = urbot;
        this.port = port;
    }

    public void start() {
        Undertow.Builder builder = Undertow.builder();
        builder.addHttpListener(port, "0.0.0.0");
        builder.setHandler(Handlers.routing()
                .post("/cmd", new HttpHandler() {
                    @Override
                    public void handleRequest(HttpServerExchange ex) throws Exception {
                        dispatch(ex, this);
                        ex.startBlocking();
                        byte[] bytes = ex.getInputStream().readAllBytes();
                        String cmd=new String(bytes);
                        urbot.sendProgram(cmd);
                    }
                })
                .post("/freedrive", new HttpHandler() {
                    @Override
                    public void handleRequest(HttpServerExchange ex) throws Exception {
                        dispatch(ex, this);
                        urbot.sendFreedrive(1);
                    }
                })
                .get("/log/{from}",  ex -> {
                    var x=ex.getQueryParameters().get("from").getFirst();
                    int from= Integer.parseInt(ex.getQueryParameters().get("from").getFirst());
                    from=Integer.max(0,from);
                    var r = URLog.get(from);
                    ex.getResponseSender().send(r.toJSON().toString());
                })
                .get("/test/{id}", ex -> {
                    System.out.println(ex.getQueryParameters().get("id").getFirst());
                })
                .get("/*", resource(new PathResourceManager(Paths.get("webroot"), 100))
                        .setDirectoryListingEnabled(true))
        );
        server = builder.build();
        server.start();
    }

    private void dispatch(HttpServerExchange ex, HttpHandler handler) {
        if (ex.isInIoThread()) {
            ex.dispatch(handler);
            return;
        }
    }
}